#! /bin/sh
# ----------------------------------------------------------------------------
# /etc/dhclient-exit-hooks - invoked by dhclient or dhcpcd
#
# Creation:     23.01.2002  od
# Last Update:  $Id: dhcpcd.sh 27516 2013-05-06 14:56:25Z kristov $
#----------------------------------------------------------------------------

dhcp_scripts ()
{
    event=$1
    logmsg dhcp-client daemon.notice "Event: $event, local: '$local', remote='$remote'"
    tty=
    speed=
    cdev=

    [ -f /var/run/dhcp-client.$interface.ip-updown ] || return

    [ $event = up ] && echo $remote > /var/run/$interface.gw

    if [ ! -f /var/run/boot_finished ]; then
	# kristov: it is important to redirect the output of this background
	# task as ip-$event will _hang_ in a loop waiting for the boot process
	# to finish; however, if there is an open pipe attached (e.g.
	# "| logmsg" below), this pipe will wait for the background task to
	# exit, which will never happen since that task waits for us to let the
	# boot process continue -- a classical deadlock
	/etc/ppp/ip-$event "$interface" "$tty" "$speed" "$local" "$remote" >/dev/null 2>&1 &
    else
	/etc/ppp/ip-$event "$interface" "$tty" "$speed" "$local" "$remote"
    fi
}

parse_info ()
{
    type=$1
    remote=''

    eval local=\$${type}_ip_address
    eval routers="\$${type}_routers"
    eval local domain_name=\$${type}_domain_name
    eval domain_name_servers="\$${type}_domain_name_servers"

    if [ "$routers" ]; then
	set -- $routers
	remote=$1
    fi

    oIFS="$IFS"
    IFS=","
    set -- $domain_name_servers
    IFS="$oIFS"

    DNS1=$1
    DNS2=$2
    DNSDOMAIN=$domain_name
    export DNS1 DNS2 DNSDOMAIN
}

run_dhcp ()
{
    logmsg dhcp-client daemon.info "DHCP event '$reason'"
    set | grep -e ^new -e ^old
    [ -f /var/run/dhcp-debug ] && set -x

    case $reason in
	BOUND|INFORM|REBOOT|TEST|TIMEOUT|IPV4LL)
	    if [ "$old_ip_address" != "$new_ip_address" -o \
		 "$new_domain_name_servers" != "$old_domain_servers" ] ; then
		if [ "$old_ip_address" ]; then
		    parse_info old
		    dhcp_scripts down
		fi
		parse_info new
		dhcp_scripts up
	    fi
	    ;;
	NOCARRIER)
	    # dhcpcd does not fill the old_... variables in a BOUND so if we
	    # have BOUND --> NOCARRIER --> CARRIER --> BOUND state changes, we
	    # have to ensure that the ip-down scripts are called
	    parse_info old
	    dhcp_scripts down
	    ;;
        REBIND|RENEW)
            def=
            last=
            [ -f /etc/default-route-interface ] && read def < /etc/default-route-interface
            [ -f /etc/last-default-route-interface ] && read last < /etc/last-default-route-interface
            if [ -f /var/run/$interface.transition ]; then
                read transition < /var/run/$interface.transition
                case $interface in
                    "$def" | "$last")
                        case $transition in
                            up)   parse_info new ;;
                            down) parse_info old ;;
                        esac
                        dhcp_scripts $transition
                        ;;
                esac
            fi
            ;;
	RELEASE | EXPIRE | TIMOUT)
	    parse_info old
	    dhcp_scripts down
	    ;;
	*)
	   logmsg dhcp-client daemon.info "    ignoring dhcp event '$reason'"
           ;;
    esac
}

# kristov: use well-defined PATH
export PATH=/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin
run_dhcp $* 2>&1 | logmsg dhcp-client daemon.debug
