#! /bin/sh

res=0

is_default()
{
    ip r s | grep -q "^default via.*dev $1"
}

error()
{
    logger -t "dhcp-circuit-ctrl.sh[$$]" $1
    res=1
}

dial()
{
    is_default $1 && return
    add-default-route $1
    echo up > /var/run/$1.transition
    dhcpcd -n $1
}

hangup()
{
    is_default $1 || return
    ip r del default dev $1
    echo down > /var/run/$1.transition
    dhcpcd -n $1
}

dialmode()
{
    case $2 in
        auto)  dial $1 ;;
        off) hangup $1 ;;
        *)   errror "unknown value '$2' for dialmode"
    esac
}

status()
{
    if is_default $1; then
        echo online
    else
        echo offline
    fi
}

logger -t dhcp-circuit-ctrl.sh "$0 $*"

if [ "$1" = -x ]; then
    exec 2> /tmp/dc.trace.$$
    set -x
    shift
fi

cmd=$1
dev=$2
arg=$3

case $cmd in
    dial)     dial     $dev ;;
    hangup)   hangup   $dev ;;
    dialmode) dialmode $dev $arg ;;
    status)   status   $dev ;;
    *) error "unsupported command $*" ;;
esac

set +x
logger -t dhcp-circuit-ctrl.sh < /tmp/dc.trace.$$
rm -f /tmp/dc.trace.$$
exit $res
